/* $Header: /home/cvsroot/MyPasswordSafe/src/encryptedstring.hpp,v 1.5.4.1 2004/02/05 03:05:27 nolan Exp $
 * Copyright (c) 2004, Semantic Gap Solutions
 * All rights reserved.
 *   
 * Redistribution and use in source and binary forms,
 * with or without modification, are permitted provided
 * that the following conditions are met:
 *  -  Redistributions of source code must retain the
 *     above copyright notice, this list of conditions
 *     and the following disclaimer.
 *  -  Redistributions in binary form must reproduce the
 *     above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or
 *     other materials provided with the distribution.
 *  -  Neither the name of Semantic Gap Solutions nor the
 *     names of its contributors may be used to endorse or
 *     promote products derived from this software without
 *     specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef ENCRYPTEDSTRING_HPP
#define ENCRYPTEDSTRING_HPP

#include <boost/shared_ptr.hpp>
#include "pwsafe/BlowFish.h"
#include "securedstring.hpp"

#if 0
class CryptoAlgorithm
{
public:
  virtual void decrypt(unsigned char *in, unsigned char *out) = 0;
  virtual void encrypt(unsigned char *in, unsigned char *out) = 0;
};

class BlowfishAlgor: public CryptoAlgorithm
{
public:
  BlowfishAlgor(const SecuredString &pass, const unsigned char *salt,
		int salt_len, unsigned char *ipthing);
  virtual ~BlowfishAlgor();

  virtual void decrypt(unsigned char *in, unsigned char *out);
  virtual void encrypt(unsigned char *in, unsigned char *out);

private:
  Blowfish *m_fish;
  unsigned char *m_ipthing;
};
#endif

class EncryptedString
{
public:
  enum Type {NAME=0, UUID=0x1, GROUP = 0x2, TITLE = 0x3, USER = 0x4, NOTES = 0x5,
	PASSWORD = 0x6, CTIME = 0x7, MTIME = 0x8, ATIME = 0x9, LTIME = 0xa,
	POLICY = 0xb, END = 0xff}; // field types, per formatV2.txt

  EncryptedString();
  EncryptedString(boost::shared_ptr<BlowFish> algor);
  EncryptedString(boost::shared_ptr<BlowFish> algor,
		  const SecuredString &str);
  EncryptedString(const SecuredString &str);
  EncryptedString(const char *str);
  EncryptedString(const EncryptedString &es);
  ~EncryptedString();

  unsigned int length() const;
  // returns the length of the string

  Type type() const;
  // returns the block type (used with Password Safe 2 files)
  void setType(const Type t);
  // sets the type

  inline boost::shared_ptr<BlowFish> getAlgorithm() const { return m_algor; }
  // returns the pointer to the BlowFish object
  void setAlgorithm(boost::shared_ptr<BlowFish> fish);
  // sets the BlowFish object pointer and re-encrypts the data

  SecuredString get() const;
  // decrypts the string and returns it in a SecuredString
  void set(const SecuredString &str);
  void set(const char *str);
  // encrypts and sets the value of the string
  void set(const EncryptedString &str);
  // encrypts the data from str using this' BlowFish object

  int read(FILE *fp, unsigned char *cbcbuffer);
  // reads a string from FP
  // cbcbuffer is the initial cbc needed to decrypt the first
  // block and is changed to the cbc needed to decrypt the
  // next entry in the safe
  // Returns the number of bytes read
  int write(FILE *fp, unsigned char *cbcbuffer);
  // writes the data to FP
  // cbcbuffer is the initial cbc used to xor the data,
  // and is set to the last encrypted block before returning
  // Returns the number of bytes written

  bool operator==(const EncryptedString &es) const;
  inline bool operator!=(const EncryptedString &es) const { return !(*this == es); }

protected:
  void trashAndDelete();
  // trashes and deletes m_data

  void createAlgorithm();
  // creates a new BlowFish object that uses a random password

  void hash();
  // hashes the unencrypted data for quick, easy, and secure comparisons
  void hash(const unsigned char *in, unsigned int in_len, unsigned char out[20]);
  // hashes in and places the hash in out

private:
  Type m_type;
  unsigned char m_cbc[8];
  unsigned char m_hash[20];
  boost::shared_ptr<BlowFish> m_algor;
  unsigned int m_length;
  unsigned char *m_data;
};

#endif
