/****************************************************************************
 ** ui.h extension file, included from the uic-generated form implementation.
 **
 ** If you wish to add, delete or rename functions or slots use
 ** Qt Designer which will update this file, preserving your code. Create an
 ** init() function in place of a constructor, and a destroy() function in
 ** place of a destructor.
 *****************************************************************************/
/* $Header: /home/cvsroot/MyPasswordSafe/src/mypasswordsafe.ui.h,v 1.35 2004/01/23 21:07:17 nolan Exp $
 * Copyright (c) 2004, Semantic Gap Solutions
 * All rights reserved.
 *   
 * Redistribution and use in source and binary forms,
 * with or without modification, are permitted provided
 * that the following conditions are met:
 *  -  Redistributions of source code must retain the
 *     above copyright notice, this list of conditions
 *     and the following disclaimer.
 *  -  Redistributions in binary form must reproduce the
 *     above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or
 *     other materials provided with the distribution.
 *  -  Neither the name of Semantic Gap Solutions nor the
 *     names of its contributors may be used to endorse or
 *     promote products derived from this software without
 *     specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <cstdlib>
#include <iostream>
#include <string>
#include <qapplication.h>
#include <qfiledialog.h>
#include <qstatusbar.h>
#include <qclipboard.h>
#include <qmessagebox.h>
#include <qsettings.h>
#include "passphrasedlg.h"
#include "pwordeditdlg.h"
#include "manualdlg.h"

using namespace std;

void MyPasswordSafe::init()
{
	savingEnabled(false);

    for(int i = 0; i < 3; i++)
	pwordListView->setColumnWidthMode(i, QListView::Manual);
    
    m_config.setPath("SemanticGap.com", "MyPasswordSafe");
    m_config.beginGroup("/MyPasswordSafe");
  m_gen_pword_length = m_config.readNumEntry("/prefs/password_length", 8);
  m_active_safe = m_config.readEntry("/prefs/active_safe", "");
  m_def_user = m_config.readEntry("/prefs/default_username", "");
  m_clear_clipboard = m_config.readBoolEntry("/prefs/clear_clipboard", true);
  m_max_tries = m_config.readNumEntry("/prefs/max_tries", 3);
  if(m_max_tries > 10)
	  m_max_tries = 10;
  
  int w, h;
  w = m_config.readNumEntry("/MainWindow/width", 400);
  h = m_config.readNumEntry("/MainWindow/height", 320);
  resize(QSize(w, h));
    
  w = m_config.readNumEntry("/MainWindow/left", -1);
  h = m_config.readNumEntry("/MainWindow/top", -1);
  if(w != -1 && h != -1) {
    move(w, h);
  }
   
  w = m_config.readNumEntry("/MainWindow/name_width", 0);
  if(w)
      pwordListView->setColumnWidth(0, w);
  w = m_config.readNumEntry("/MainWindow/user_width", 0);
  if(w)
      pwordListView->setColumnWidth(1, w);
  w = m_config.readNumEntry("/MainWindow/notes_width", 0);
  if(w)
      pwordListView->setColumnWidth(2, w);
  
  m_config.endGroup();
  
  m_safe = NULL;
  if(m_active_safe.length() > 0) {
    if(open(m_active_safe) == true) {
      statusBar()->message(tr("Active safe opened"));
    }
  }
    
  if(m_safe == NULL) {
    //m_safe = new Safe();
    //m_safe->setListBox(pwordListView);
	  fileNew();
  }
}

void MyPasswordSafe::destroy()
{
  // ask to save file if needed
  if(m_clear_clipboard)
    QApplication::clipboard()->setText("");
  // save config settings   
  m_config.beginGroup("/MyPasswordSafe");
  
  m_config.writeEntry("/prefs/password_length", (int)m_gen_pword_length);
  m_config.writeEntry("/prefs/active_safe", m_active_safe);
  m_config.writeEntry("/prefs/default_username", m_def_user);
  m_config.writeEntry("/prefs/clear_clipboard", m_clear_clipboard);
  m_config.writeEntry("/prefs/max_tries", m_max_tries);
  
  QSize sz = size();
  m_config.writeEntry("/MainWindow/width", sz.width());
  m_config.writeEntry("/MainWindow/height", sz.height());

  QPoint position = pos();
  m_config.writeEntry("/MainWindow/left", position.x());
  m_config.writeEntry("/MainWindow/top", position.y());
 
  m_config.writeEntry("/MainWindow/name_width", pwordListView->columnWidth(0));
  m_config.writeEntry("/MainWindow/user_width", pwordListView->columnWidth(1));
  m_config.writeEntry("/MainWindow/notes_width", pwordListView->columnWidth(2));
  m_config.endGroup();
  
  if(m_safe)
      delete m_safe;
}

void MyPasswordSafe::fileNew()
{
    if(closeSafe()) {
	m_safe = new Safe();
	m_safe->setListBox(pwordListView);
	setCaption(tr("MyPasswordSafe: <untitled>"));
	savingEnabled(false);
	fileSaveAsAction->setEnabled(false);
    }
}


void MyPasswordSafe::fileOpen()
{
  QString types(SafeSerializer::getTypes().c_str());
  QFileDialog file_dlg(QString::null, types, this, "file open",
		       true);
    
  if(file_dlg.exec() == QDialog::Accepted) {
    QString filename = file_dlg.selectedFile();
    if(!filename.isEmpty()) {
      QString filter = file_dlg.selectedFilter();
      QString file_ext = filename.right(filename.findRev('.'));
      const char *ext = SafeSerializer::getExtForType(filter);
      bool open_ret = false;
      if(file_ext == ext) {
	open_ret = open(filename);
      }
      else {
	open_ret = open(filename, ext);
      }
      
      if(open_ret) {
	statusBar()->message(tr("Safe opened"));
      }
      else {
	statusBar()->message(tr("Failed to open safe"));
      }
    }
  }
  else {
    statusBar()->message(tr("Open file cancelled"));
  }
}


void MyPasswordSafe::fileSave()
{
    if(m_safe != NULL) {
	if(!m_safe->save((const char *)m_def_user)) {
	    fileSaveAs();
	}
	savingEnabled(false);
    }
}


void MyPasswordSafe::fileSaveAs()
{
  saveAs();
}

bool MyPasswordSafe::saveAs()
{
  QFileDialog file_dlg(QString::null, SafeSerializer::getTypes().c_str(), this, "file save",
		       true);
  file_dlg.setMode(QFileDialog::AnyFile);
    
  if(file_dlg.exec() == QDialog::Accepted) {
    QString filename = file_dlg.selectedFile();

    int dot = filename.findRev('.');
    int slash = filename.findRev('/');

    if(dot < slash) {
      // FIXME: if there is a file extension, but the filter
      // is different what do we do? Use the file's extension
      // or the filter? Currently it's based off of the file's
      // extension.

      // we need to come up with some file extension
      QString filter = file_dlg.selectedFilter();
      const char *ext = SafeSerializer::getExtForType(filter);
#ifdef DEBUG
      cout << "Filter:\t" << filter << endl
	   << "Extension:\t" << ext << endl;
#endif
      if(ext == NULL) {
	statusBar()->message(tr("Unknown file type, save aborted"));
	return false;
      }
      else
	filename = filename + "." + ext;
    }

    //QString filter = file_dlg.selectedFilter();
    if(!filename.isEmpty()) {
      // NOTE: the passphrase doesn't get encrypted here
      SecuredString passkey;
      PassPhraseDlg dlg;
      if(dlg.exec() == QDialog::Accepted) {
	passkey.set(dlg.getText());
	if(m_safe->save((const char *)filename,
			passkey, (const char *)m_def_user)) {
	    setCaption(tr("MyPasswordSafe: ") + filename);
	  statusBar()->message(tr("Safe saved"));
	  savingEnabled(false);
	  return true;
	}
	statusBar()->message(tr("Error saving file"));
      }
      else {
	statusBar()->message(tr("No passkey entered"));
      }
    }
  }
  else {
    statusBar()->message(tr("Save file cancelled"));
  }
  return false;
}



void MyPasswordSafe::pwordAdd()
{
  PwordEditDlg dlg;
  dlg.setGenPwordLength(m_gen_pword_length);
  dlg.setUser(QString::fromUtf8(m_def_user));
  if(dlg.exec() == QDialog::Accepted) {
    SafeItem i((const char *)dlg.getItemName().utf8(),
	       (const char *)dlg.getUser().utf8(),
	       SecuredString((const char *)dlg.getPassword().utf8()),
	       (const char *)dlg.getNotes().utf8());
    m_safe->addItem(i);
    savingEnabled(true);
  }
}

void MyPasswordSafe::helpAbout()
{
  AboutDlg dlg;
  dlg.exec();
}


void MyPasswordSafe::pwordDelete()
{
  QListViewItem *list_item(pwordListView->selectedItem());
  if(list_item) {
    int result = QMessageBox::warning(this, tr("Are you sure?"),
				      tr("Are you sure you want to delete this password?"),
				      QMessageBox::Yes, QMessageBox::No);
    switch(result) {
    case QMessageBox::Yes:
      m_safe->delItem(list_item);
      savingEnabled(true);
      statusBar()->message(tr("Password deleted"));
      break;
    default:
      statusBar()->message(tr("Delete cancelled"));
      break;
    }
  }
  else {
      statusBar()->message(tr("No item selected"));
  }
}


void MyPasswordSafe::pwordEdit()
{
  QListViewItem *list_item = pwordListView->selectedItem();
  SafeItem *item = m_safe->findItem(list_item);
  if(item != NULL) {
    PwordEditDlg dlg;
    dlg.setGenPwordLength(m_gen_pword_length);
    dlg.setItemName(QString::fromUtf8(item->getName()));
    dlg.setUser(QString::fromUtf8(item->getUser()));
    // NOTE: password decrypted
    dlg.setPassword(QString::fromUtf8(item->getPassword().get().get()));
    dlg.setNotes(QString::fromUtf8(item->getNotes()));
    if(dlg.exec() == QDialog::Accepted) {
      item->setName((const char *)dlg.getItemName().utf8());
      item->setUser((const char *)dlg.getUser().utf8());
      item->setPassword((const char *)dlg.getPassword().utf8());
      item->setNotes((const char *)dlg.getNotes().utf8());
      item->updateListItem(list_item);
      m_safe->setChanged(true);
      savingEnabled(true);
      statusBar()->message(tr("Password updated"));
  }
  else {
    statusBar()->message(tr("Edit password cancelled"));
    }
  }
  else {
    statusBar()->message(tr("No item selected"));
  }
}


void MyPasswordSafe::pwordFetch()
{
  QListViewItem *list_item(pwordListView->selectedItem());
  if(list_item) {
    SafeItem *item = m_safe->findItem(list_item);
    if(item) {
      QClipboard *cb = QApplication::clipboard();
      // NOTE: password decrypted
      SecuredString pword(item->getPassword().get());
      cb->setText(QString::fromUtf8(pword.get()));
      statusBar()->message(tr("Password copied to clipboard"));
    }
    else {
      statusBar()->message(tr("Invalid item selected??"));
    }
  }
  else {
    statusBar()->message(tr("No item selected"));
  }
}

void MyPasswordSafe::fileExit()
{
    //if(closeSafe()) {
	close();
    //}
}


void MyPasswordSafe::fileMakeActive()
{
  string path(m_safe->getPath());
  if(path.empty()) {
    if(!saveAs()) {
      statusBar()->message(tr("The safe must be saved before it can be made active."));
	  return;
    }
  }
  m_active_safe = m_safe->getPath().c_str();
}


void MyPasswordSafe::filePreferences()
{
  PreferencesDlg dlg;
  dlg.setDefUser(m_def_user);
  dlg.setActiveSafe(m_active_safe);
  dlg.setClearClipboard(m_clear_clipboard);
  dlg.setGenPwordLength(m_gen_pword_length);
  dlg.setMaxTries(m_max_tries);
  if(dlg.exec() == QDialog::Accepted) {
    m_def_user = dlg.getDefUser();
    m_active_safe = dlg.getActiveSafe();
    m_clear_clipboard = dlg.getClearClipboard();
    m_gen_pword_length = dlg.getGenPwordLength();
	m_max_tries = dlg.getMaxTries();
  }
}


void MyPasswordSafe::onPwordListDblClicked( QListViewItem * )
{
  pwordFetch();
}


void MyPasswordSafe::onPwordListRenamed( QListViewItem *item, int, const QString & )
{
}


void MyPasswordSafe::onPwordListRightClk( QListViewItem *item, const QPoint &point, int index)
{
  PopupMenu->popup(point, index);
}

Safe * MyPasswordSafe::getSafe()
{
  return m_safe;
}


int MyPasswordSafe::getGeneratedPwordLength()
{
  return m_gen_pword_length;
}

bool MyPasswordSafe::open(const char *filename)
{
	return open(filename, NULL);
}

bool MyPasswordSafe::open( const char *filename, const char *ext )
{
  SecuredString passkey;
  int num_tries = 0;
  PassPhraseDlg dlg;
  while(num_tries < m_max_tries) {
    if(dlg.exec() == QDialog::Accepted) {
      passkey.set(dlg.getText());
      bool ret = false;
      if(ext != NULL)
	ret = Safe::checkPassword(filename, ext, passkey);
      else
	ret = Safe::checkPassword(filename, passkey);
      if(ret == true) {
	// this is when we ask the user if they want to save a
	// modified safe or not
	if(!closeSafe())
	  return false;
	Safe *s = NULL;
	if(ext != NULL)
	  s = Safe::load(filename, ext, passkey,
			 (const char *)m_def_user,
			 pwordListView);
	else
	  s = Safe::load(filename, passkey,
			 (const char *)m_def_user,
			 pwordListView);
	if(s != NULL) {
	  if(m_safe != NULL)
	    delete m_safe;
	  m_safe = s;
	  //m_safe->setListBox(pwordListView);
	  setCaption(tr("MyPasswordSafe: ") + filename);
	  savingEnabled(false);
	  fileSaveAsAction->setEnabled(true);
	  return true;
	}
	else {
	  statusBar()->message(tr("Error opening file"));
	  break;
	}
      }
      else {
	// FIXME: could we distinguish what the error was?
	statusBar()->message(tr("Wrong pass phrase or unsupported file format"));
      }
    }
    else {
      statusBar()->message(tr("No passkey entered"));
      break;
    }
    num_tries++;
  }
  return false;
}


bool MyPasswordSafe::closeSafe()
{
  if(m_safe) {
    if(m_safe->changed()) {
      int result = QMessageBox::warning(this, tr("Save safe?"),
					tr("Do you want to save the safe before it is closed?"),
					QMessageBox::Yes, QMessageBox::No, QMessageBox::Cancel);
      switch(result) {
      case QMessageBox::Yes:
	fileSave();
	break;
      case QMessageBox::Cancel:
	return false;
      default:
	break;
      }
    }
    delete m_safe;
    m_safe = NULL;
    savingEnabled(false);
  }
  return true;
}

void MyPasswordSafe::closeEvent( QCloseEvent *e )
{
    if(closeSafe()) {
	e->accept();
    }
}


void MyPasswordSafe::savingEnabled( bool value)
{
	if(value) {
		fileSaveAction->setEnabled(true);
		//fileSaveAsAction->setEnabled(true);
	}
	else {
		fileSaveAction->setEnabled(false);
	}
}


void MyPasswordSafe::pwordFetchUser()
{
  QListViewItem *list_item(pwordListView->selectedItem());
  if(list_item) {
    SafeItem *item = m_safe->findItem(list_item);
    if(item) {
      QClipboard *cb = QApplication::clipboard();
      cb->setText(QString::fromUtf8(item->getUser()));
      statusBar()->message(tr("Username copied to clipboard"));
    }
	else {
		statusBar()->message(tr("Invalid item selected??"));
	}
  }
  else {
      statusBar()->message(tr("No item selected"));
  }
}


void MyPasswordSafe::helpManual()
{
	if(m_manual.isVisible()) {
		m_manual.hide();
	}
	else {
		m_manual.show();
	}
}
