// pws_platform.h
//------------------------------------------------------------------------------
//
// This header file exists to determine, at compile time, which target we're
// compiling to and to make the appropriate #defines and #includes.
//
// The following macros are defined:
//
//    BIG_ENDIAN       - Defined only if the target CPU is big-endian.
//    LITTLE_ENDIAN    - Defined only if the target CPU is little-endian.
//    PWS_PLATFORM     - A string, the target platform, e.g. "Pocket PC".
//    PWS_PLATFORM_EX  - A string, the target platform, e.g. "Pocket PC 2000".
//    POCKET_PC        - Defined only if target is Pocket PC 2000 or later.
//    POCKET_PC_VER    - Defined only if target is Pocket PC 2000 or later.
//
// Notes:
//
// 1. BIG_ENDIAN and LITTLE_ENDIAN are mutually exclusive.
// 2. BIG_ENDIAN and LITTLE_ENDIAN may be defined on the complier command line
//    but checks are made to ensure that one and only one is defined.
//
// Supported Configurations:
// -------------------------
//
// Pocket PC:
//
//    2000 ARM
//    2000 MIPS
//    2000 SH3
//    2000 x86 Emulator
//    2002 ARM
//    2002 x86 Emulator
//    2003 ARM (untested)
//    2003 x86 Emulator (untested)
//
// Windows
//
//    Win32 X86

#ifndef PwsPlatform_h
#define PwsPlatform_h

#if WIN32
#define __LITTLE_ENDIAN 1234
#define __BIG_ENDIAN 4321
#define __BYTE_ORDER __LITTLE_ENDIAN
#endif

#endif
