/* $Header: /home/cvsroot/MyPasswordSafe/src/serializers.hpp,v 1.10 2004/01/18 06:46:24 nolan Exp $
 * Copyright (c) 2004, Semantic Gap Solutions
 * All rights reserved.
 *   
 * Redistribution and use in source and binary forms,
 * with or without modification, are permitted provided
 * that the following conditions are met:
 *  -  Redistributions of source code must retain the
 *     above copyright notice, this list of conditions
 *     and the following disclaimer.
 *  -  Redistributions in binary form must reproduce the
 *     above copyright notice, this list of conditions and
 *     the following disclaimer in the documentation and/or
 *     other materials provided with the distribution.
 *  -  Neither the name of Semantic Gap Solutions nor the
 *     names of its contributors may be used to endorse or
 *     promote products derived from this software without
 *     specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef _SERIALIZERS_HPP_
#define _SERIALIZERS_HPP_

#include "safe.hpp"
#include "pwsafe/Util.h"

/* Instances of these classes are in safe.cpp after the SerializerMap.
 */

class PlainTextLizer: public SafeSerializer
{
public:
  PlainTextLizer();

  virtual const char *description();

  virtual bool checkPassword(const string &path, const SecuredString &password);
  virtual bool load(Safe &safe, const string &path, const EncryptedString &passphrase, const string &def_user);
  virtual bool save(Safe &safe, const string &path, const string &def_user);
};

class BlowfishLizer: public SafeSerializer
{
public:
  BlowfishLizer();

  virtual const char *description();

  virtual bool checkPassword(const string &path, const SecuredString &password);
  virtual bool load(Safe &safe, const string &path, const EncryptedString &passphrase, const string &def_user);
  virtual bool save(Safe &safe, const string &path, const string &def_user);

protected:
  virtual int readHeader(FILE *in, unsigned char randstuff[8],
		 unsigned char randhash[20],
		 unsigned char salt[SaltLength],
		 unsigned char ipthing[8]);
  virtual int readEntry(FILE *in, SafeItem &item, EncryptedString &es,
		unsigned char *ipthing, const string &def_user);

private:
  int writeCBC(FILE *fp, string &data, const SecuredString &passkey,
	       const unsigned char *salt, unsigned char *ipthing);
  int readCBC(FILE *fp, string &data, const SecuredString &passkey,
	      const unsigned char *salt, unsigned char *ipthing);
};

#endif
